@@ -4,9 +4,11 @@ import textwrap
 import copy
 
 from pathlib import Path
+from pkg_resources import Requirement
 
-from .. import files
-from .component import Component
+from cirrus.core.exceptions import DuplicateRequirement, MissingHandler
+from cirrus.core.components import files
+from cirrus.core.components.base import Component
 from cirrus.core.utils.yaml import NamedYamlable
 
 
@@ -14,10 +16,11 @@ logger = logging.getLogger(__name__)
 
 
 class Lambda(Component):
-    handler = files.PythonHandler()
+    handler = files.PythonHandler(optional=True)
     definition = files.LambdaDefinition()
     # TODO: Readme should be required once we have one per task
     readme = files.Readme(optional=True)
+    user_extendable = False
 
     def load_config(self):
         super().load_config()
@@ -28,7 +31,11 @@ class Lambda(Component):
         self.environment = self.config.get('environment', NamedYamlable())
 
         self.lambda_config = self.config.get('lambda', NamedYamlable())
-        self.lambda_enabled = self.lambda_config.pop('enabled', True) and self._enabled and bool(self.lambda_config)
+        self.lambda_enabled = (
+            bool(self.lambda_config)
+            and self.lambda_config.pop('enabled', True)
+            and self._enabled
+        )
         self.lambda_config.description = self.description
 
         project_reqs = []
@@ -52,9 +59,8 @@ class Lambda(Component):
 
         if not hasattr(self.lambda_config, 'pythonRequirements'):
             self.lambda_config.pythonRequirements = {}
-        # note the set to deduplicate requirements
-        # TODO: multiple versions of the same requirement
-        # will not be deduplicated
+        # note the set to deduplicate requirements but that different
+        # versions/pinning for the same package will not be deduplicated
         self.lambda_config.pythonRequirements['include'] = sorted(list({
             req for req in
             # list of all requirements specified in lambda config
@@ -63,10 +69,25 @@ class Lambda(Component):
             + project_reqs
         }))
 
-        if not hasattr(self.lambda_config, 'module'):
+        if hasattr(self.lambda_config, 'handler'):
+            # this is a non-container lambda that needs to be packaged
+            # by serverless, then we need to ensure the module points to the
+            # place in the build dir where we'll copy all the code
             self.lambda_config.module = f'lambdas/{self.name}'
-        if not hasattr(self.lambda_config, 'handler'):
-            self.lambda_config.handler = 'lambda_function.lambda_handler'
+            self.handler.path = Path(self.lambda_config.handler).with_suffix('.py')
+            self.handler.validate(required=self.lambda_enabled)
+        elif not hasattr(self.lambda_config, 'image') and self.enabled:
+            # this is also not a container lambda, which means it is likely
+            # a lambda that has not been updated to have an explicit handler
+            raise MissingHandler(
+                'Missing module parameter in lambda definiton.yml. '
+                'The handler parameter, which sets the module, is no longer defaulted. '
+                'You likely need to set it to be like:\n\n'
+                '    lambda:\n      handler: lambda_function.lambda_handler\n\n'
+                f'Offending {self.type}: {self.name}\n\n'
+                'See https://github.com/cirrus-geo/cirrus-geo/issues/139 for additional context.'
+            ) from None
+
 
     @property
     def enabled(self):
@@ -92,11 +113,16 @@ class Lambda(Component):
         return lc
 
     def get_outdir(self, project_build_dir: Path) -> Path:
+        if not self.lambda_enabled or not hasattr(self.lambda_config, 'module'):
+            return None
         return project_build_dir.joinpath(self.lambda_config.module)
 
     def copy_to_outdir(self, outdir: Path) -> None:
         import shutil
 
+        if not self.lambda_enabled:
+            return
+
         try:
             outdir.mkdir(parents=True)
         except FileExistsError:
@@ -115,12 +141,27 @@ class Lambda(Component):
             else:
                 shutil.copyfile(_file, outdir.joinpath(_file.name))
 
-        outdir.joinpath('requirements.txt').write_text(
-            ''.join(
-                [f'{req}\n' for req in
-                 self.lambda_config.pythonRequirements.get('include', [])],
-            ),
-        )
+        requirements = self.lambda_config.pythonRequirements.get('include', [])
+        requirements_str = '\n'.join(requirements)
+
+        # if we have multiple versions specified for the same requirement
+        # we want to throw a meaningful error, so we parse them all here
+        # and check for dups
+        req_names = set()
+        for req in requirements:
+            req = Requirement.parse(req)
+            if req.name in req_names:
+                raise DuplicateRequirement(
+                    f"ERROR: Duplicated requirement for {self.type} '{self.name}', package '{req.name}'.\n\n"
+                    'At this time cirrus does not support verison conflict resolution.\n'
+                    'Please review the requirements keeping in mind those for cirrus-lib.\n'
+                    'See https://github.com/cirrus-geo/cirrus-geo/issues/106 for context.\n\n'
+                    f'Full requirements list:\n{requirements_str}'
+                )
+            else:
+                req_names.add(req.name)
+
+        outdir.joinpath('requirements.txt').write_text(requirements_str + '\n')
 
     def clean_outdir(self, outdir: Path):
         import shutil
